"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.instanceConfigFields = void 0;
exports.validateInvocation = validateInvocation;
const integration_sdk_core_1 = require("@jupiterone/integration-sdk-core");
const provider_1 = require("./provider");
exports.instanceConfigFields = {
    personalToken: {
        type: 'string',
        mask: true,
    },
    baseUrl: {
        type: 'string',
    },
    mergeRequestsUpdatedAfter: {
        type: 'string',
        optional: true,
    },
    vulnerabilitySeverities: {
        type: 'string',
        optional: true,
    },
    vulnerabilityStates: {
        type: 'string',
        optional: true,
    },
    vulnerabilityReportTypes: {
        type: 'string',
        optional: true,
    },
};
const SEVERITIES = ['critical', 'high', 'medium', 'low', 'info'];
const STATES = ['confirmed', 'detected', 'dismissed', 'resolved'];
const REPORT_TYPES = [
    'api_fuzzing',
    'cluster_image_scanning',
    'container_scanning',
    'container_scanning_for_registry',
    'coverage_fuzzing',
    'dast',
    'dependency_scanning',
    'generic',
    'sast',
    'secret_detection',
];
function validSeverities(severities) {
    return severities.every((severity) => SEVERITIES.includes(severity));
}
function validStates(states) {
    return states.every((state) => STATES.includes(state));
}
function validReportTypes(reportTypes) {
    return reportTypes.every((reportType) => REPORT_TYPES.includes(reportType));
}
async function validateInvocation(context) {
    const { config } = context.instance;
    if (!config.baseUrl || !config.personalToken) {
        throw new integration_sdk_core_1.IntegrationValidationError('Config requires all of {baseUrl, personalToken}');
    }
    if (config.mergeRequestsUpdatedAfter) {
        config.mergeRequestsUpdatedAfter = new Date(config.mergeRequestsUpdatedAfter);
    }
    else {
        const sevenDaysAgo = new Date();
        sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);
        config.mergeRequestsUpdatedAfter = sevenDaysAgo;
    }
    if (config.vulnerabilitySeverities) {
        const severities = config.vulnerabilitySeverities
            .split(',')
            .map((s) => s.trim());
        if (!validSeverities(severities)) {
            throw new integration_sdk_core_1.IntegrationValidationError(`Invalid vulnerability severities. Valid severities are ${SEVERITIES.join(', ')}`);
        }
        config.vulnerabilitySeverities = severities;
    }
    else {
        config.vulnerabilitySeverities = undefined;
    }
    if (config.vulnerabilityStates) {
        const states = config.vulnerabilityStates
            .split(',')
            .map((s) => s.trim());
        if (!validStates(states)) {
            throw new integration_sdk_core_1.IntegrationValidationError(`Invalid vulnerability states. Valid states are ${STATES.join(', ')}`);
        }
        config.vulnerabilityStates = states.map((s) => s.toUpperCase());
    }
    else {
        config.vulnerabilityStates = undefined;
    }
    if (config.vulnerabilityReportTypes) {
        const reportTypes = config.vulnerabilityReportTypes
            .split(',')
            .map((s) => s.trim());
        if (!validReportTypes(reportTypes)) {
            throw new integration_sdk_core_1.IntegrationValidationError(`Invalid vulnerability report types. Valid report types are ${REPORT_TYPES.join(', ')}`);
        }
        config.vulnerabilityReportTypes = reportTypes;
    }
    else {
        config.vulnerabilityReportTypes = undefined;
    }
    const apiClient = (0, provider_1.createAPIClient)(config, context.logger);
    await apiClient.verifyAuthentication();
}
//# sourceMappingURL=config.js.map