"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphQLClient = void 0;
const tslib_1 = require("tslib");
const integration_sdk_core_1 = require("@jupiterone/integration-sdk-core");
const graphqlQueries_1 = require("./graphqlQueries");
const integration_sdk_http_client_1 = require("@jupiterone/integration-sdk-http-client");
const to_json_schema_1 = tslib_1.__importDefault(require("to-json-schema"));
const errors_1 = require("./errors");
const promises_1 = require("timers/promises");
const PAGE_SIZE = 100;
function getQueryName(queryString) {
    const match = queryString.match(/query\s+(\w+)/);
    return match ? match[1] : null;
}
class GraphQLClient extends integration_sdk_http_client_1.BaseAPIClient {
    personalToken;
    constructor(config, logger) {
        super({
            baseUrl: config.baseUrl,
            logger,
            rateLimitThrottling: {
                threshold: 0.1,
                resetMode: 'datetime_epoch_s',
                rateLimitHeaders: {
                    limit: 'RateLimit-Limit',
                    remaining: 'RateLimit-Remaining',
                    reset: 'RateLimit-Reset',
                },
            },
        });
        this.personalToken = config.personalToken;
        this.retryOptions.handleError = this.getHandleErrorFn();
    }
    getHandleErrorFn() {
        return async (err, context, logger) => {
            if (['ECONNRESET', 'ETIMEDOUT', 'EAI_AGAIN', 'ECONNREFUSED'].some((code) => err.code === code || err.message.includes(code))) {
                return;
            }
            if (!err.retryable) {
                // can't retry this? just abort
                context.abort();
                return;
            }
            if (err.status === 429) {
                const retryAfter = err.retryAfter ? err.retryAfter * 1000 : 5_000;
                logger.warn({ retryAfter }, 'Received a rate limit error. Waiting before retrying.');
                await (0, promises_1.setTimeout)(retryAfter);
            }
        };
    }
    getAuthorizationHeaders() {
        return {
            'Private-Token': this.personalToken,
        };
    }
    async gql(query, variables) {
        const doRequest = async (retryCount = 0) => {
            const response = await this.retryableRequest(`/api/graphql`, {
                method: 'POST',
                body: {
                    query,
                    variables,
                },
            });
            const data = await response.json();
            if (data.errors?.length) {
                const queryName = getQueryName(query);
                this.logger.warn({
                    queryName,
                    variables,
                    gqlErrors: `GraphQL Errors: ${JSON.stringify(data.errors)}`,
                }, `Errors returned from the Gitlab GraphQL API in the response for query "${queryName}"`);
                if (data.errors.some((e) => e.message === 'Timeout on validation of query')) {
                    if (retryCount < 2) {
                        // only retry twice
                        await (0, promises_1.setTimeout)(30_000);
                        this.logger.info({ queryName, variables, retryCount: retryCount + 1 }, 'Retrying query');
                        return doRequest(retryCount + 1);
                    }
                    else {
                        throw new integration_sdk_core_1.IntegrationProviderAPIError({
                            endpoint: this.withBaseUrl(`/graphql`),
                            status: data.errors[0].extensions?.code ?? 'UNKNOWN',
                            statusText: data.errors
                                .map((e) => e.message)
                                .join(', '),
                        });
                    }
                }
            }
            return { data: data.data, errors: data.errors };
        };
        return doRequest();
    }
    async iteratePages(query, variables, extractConnection, iteratee, handleError) {
        let cursor;
        let pageSize = variables.first ?? PAGE_SIZE;
        let retry = false;
        do {
            retry = false;
            const completeVariables = {
                cursor,
                ...variables,
                first: pageSize,
            };
            let response;
            try {
                response = await this.gql(query, completeVariables);
            }
            catch (err) {
                if (err.cause.bodyError.includes('Request timed out. Please try a less complex query or a smaller set of records.')) {
                    const newPageSize = Math.max(Math.floor(pageSize / 2), 1);
                    if (newPageSize === pageSize) {
                        // Can't reduce page size any further, throw error
                        throw err;
                    }
                    pageSize = newPageSize;
                    retry = true;
                    continue;
                }
                else {
                    throw err;
                }
            }
            const { data, errors } = response;
            const connection = extractConnection(data);
            if (!connection?.nodes) {
                let dataSchema;
                try {
                    dataSchema = JSON.stringify((0, to_json_schema_1.default)(data));
                }
                catch (err) {
                    // Ignore error.
                }
                this.logger.warn({
                    queryName: getQueryName(query),
                    variables: completeVariables,
                    dataSchema,
                    gqlErrors: `GraphQL Errors: ${JSON.stringify(errors)}`,
                }, 'Expected a valid response and is null or undefined');
            }
            if (errors?.length && handleError) {
                handleError(new errors_1.GraphQLError({
                    endpoint: this.withBaseUrl(`/graphql`),
                    errors,
                }));
            }
            for (const item of connection?.nodes ?? []) {
                await iteratee(item);
            }
            if (connection?.pageInfo?.hasNextPage) {
                cursor = connection.pageInfo.endCursor ?? undefined;
            }
            else {
                cursor = undefined;
            }
        } while (cursor || retry);
    }
    async verifyAuthentication() {
        try {
            await this.fetchTokenOwner();
        }
        catch (err) {
            throw new integration_sdk_core_1.IntegrationProviderAuthenticationError({
                cause: err,
                endpoint: this.withBaseUrl(`/api/graphql`),
                status: err.status,
                statusText: err.statusText,
            });
        }
    }
    async fetchTokenOwner() {
        const { data } = await this.gql(graphqlQueries_1.TokenOwnerQuery);
        return data.currentUser;
    }
    async fetchSystemVersion() {
        const { data } = await this.gql(graphqlQueries_1.SystemVersionQuery);
        return data.metadata;
    }
    async iterateGroups(iteratee) {
        await this.iteratePages(graphqlQueries_1.GroupsQuery, {}, (data) => {
            return data?.currentUser?.groups;
        }, iteratee);
    }
    async iterateProjects(iteratee) {
        await this.iteratePages(graphqlQueries_1.ProjectsQuery, {}, (data) => {
            return data?.projects;
        }, iteratee);
    }
    async iterateGroupMembers(groupFullPath, iteratee) {
        await this.iteratePages(graphqlQueries_1.GroupMembersQuery, { groupFullPath }, (data) => {
            return data?.group?.groupMembers;
        }, iteratee);
    }
    async iterateProjectMembers(projectFullPath, iteratee) {
        await this.iteratePages(graphqlQueries_1.ProjectMembersQuery, { projectFullPath }, (data) => {
            return data?.project?.projectMembers;
        }, iteratee);
    }
    async iterateGroupVulnerabilities(groupFullPath, vulnerabilityFilters, iteratee) {
        await this.iteratePages(graphqlQueries_1.GroupVulnerabilitiesQuery, {
            groupFullPath,
            severities: (vulnerabilityFilters.severities ?? ['critical', 'high', 'medium']).map((severity) => severity.toUpperCase()),
            states: vulnerabilityFilters.states?.map((state) => state.toUpperCase()),
            reportTypes: vulnerabilityFilters.reportTypes?.map((reportType) => reportType.toUpperCase()),
        }, (data) => {
            return data?.group?.vulnerabilities;
        }, iteratee);
    }
    async iterateProjectLabels(projectFullPath, iteratee, handleError) {
        await this.iteratePages(graphqlQueries_1.ProjectLabelsQuery, { projectFullPath }, (data) => {
            return data?.project?.labels;
        }, iteratee, handleError);
    }
    iterateProjectMergeRequests(projectFullPath, updatedAfter, iteratee) {
        return this.iteratePages(graphqlQueries_1.ProjectMergeRequestsQuery, { projectFullPath, updatedAfter }, (data) => {
            return data?.project?.mergeRequests;
        }, iteratee);
    }
    async iterateMergeRequestCommits(mergeRequestId, iteratee) {
        await this.iteratePages(graphqlQueries_1.MergeRequestCommitsQuery, { mergeRequestId }, (data) => {
            return data?.mergeRequest?.commits;
        }, iteratee);
    }
    async iterateProjectBranchRules(projectFullPath, iteratee) {
        await this.iteratePages(graphqlQueries_1.ProjectBranchRulesQuery, { projectFullPath }, (data) => {
            return data?.project?.branchRules;
        }, iteratee);
    }
}
exports.GraphQLClient = GraphQLClient;
//# sourceMappingURL=GraphQLClient.js.map