"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeRequestSteps = void 0;
exports.fetchMergeRequests = fetchMergeRequests;
const integration_sdk_core_1 = require("@jupiterone/integration-sdk-core");
const constants_1 = require("../../constants");
const provider_1 = require("../../provider");
const converter_1 = require("../users/converter");
const converter_2 = require("./converter");
const getNumericalId_1 = require("../../util/getNumericalId");
async function fetchMergeRequests({ logger, instance, jobState, }) {
    const updatedAfter = instance.config.mergeRequestsUpdatedAfter;
    if (!updatedAfter) {
        throw new integration_sdk_core_1.IntegrationConfigLoadError('Missing config.mergeRequestsUpdatedAfter!');
    }
    const client = (0, provider_1.createAPIClient)(instance.config, logger);
    let totalProjectsProcessed = 0;
    let totalErrors = 0;
    const unauthorizedProjectIds = new Set();
    await jobState.iterateEntities(constants_1.Entities.PROJECT, async (projectEntity) => {
        const project = (0, integration_sdk_core_1.getRawData)(projectEntity);
        if (!project) {
            throw new integration_sdk_core_1.IntegrationError({
                code: 'RAW_DATA_MISSING',
                message: `Could not find Project entity with _key ${projectEntity._key}`,
            });
        }
        try {
            await client.iterateProjectMergeRequests(project.fullPath, updatedAfter.toISOString(), async (mergeRequest) => {
                const mergeRequestEntity = await jobState.addEntity((0, converter_2.createMergeRequestEntity)(project.name, mergeRequest));
                await jobState.addRelationship((0, integration_sdk_core_1.createDirectRelationship)({
                    _class: integration_sdk_core_1.RelationshipClass.HAS,
                    from: projectEntity,
                    to: mergeRequestEntity,
                }));
                const openedUserKey = mergeRequest.author?.id &&
                    (0, converter_1.createUserEntityIdentifier)((0, getNumericalId_1.getNumericalId)(mergeRequest.author.id));
                if (openedUserKey && jobState.hasKey(openedUserKey)) {
                    await jobState.addRelationship((0, integration_sdk_core_1.createDirectRelationship)({
                        _class: integration_sdk_core_1.RelationshipClass.OPENED,
                        fromKey: openedUserKey,
                        fromType: constants_1.Entities.USER._type,
                        toKey: mergeRequestEntity._key,
                        toType: constants_1.Entities.MERGE_REQUEST._type,
                    }));
                }
                for (const approver of mergeRequest.approvedBy?.nodes ?? []) {
                    if (!approver) {
                        continue;
                    }
                    const userEntityKey = (0, converter_1.createUserEntityIdentifier)((0, getNumericalId_1.getNumericalId)(approver.id));
                    if (jobState.hasKey(userEntityKey)) {
                        await jobState.addRelationship((0, integration_sdk_core_1.createDirectRelationship)({
                            _class: integration_sdk_core_1.RelationshipClass.APPROVED,
                            fromKey: userEntityKey,
                            fromType: constants_1.Entities.USER._type,
                            toKey: mergeRequestEntity._key,
                            toType: constants_1.Entities.MERGE_REQUEST._type,
                        }));
                    }
                    else {
                        logger.warn({ mergeRequestId: mergeRequest.id, userEntityKey }, 'Failed to locate user entity for merge request approval');
                    }
                }
            });
        }
        catch (err) {
            const logDetail = {
                err,
                projectId: project.id,
            };
            if (err instanceof integration_sdk_core_1.IntegrationProviderAuthorizationError) {
                logger.warn(logDetail, 'Unauthorized to fetch merge requests for project');
                unauthorizedProjectIds.add(project.id);
            }
            else {
                logger.error(logDetail, 'Failed to fetch merge requests for project');
            }
            totalErrors++;
        }
        totalProjectsProcessed++;
    });
    logger.publishInfoEvent({
        name: integration_sdk_core_1.IntegrationInfoEventName.Stats,
        description: `Processed merge requests for ${totalProjectsProcessed} projects${totalErrors > 0
            ? `, errors for ${totalErrors} projects${unauthorizedProjectIds.size > 0
                ? `(${unauthorizedProjectIds.size} unauthorized)`
                : ''} projects`
            : ''}`,
    });
}
exports.mergeRequestSteps = [
    {
        id: constants_1.Steps.MERGE_REQUESTS,
        name: 'Fetch merge requests',
        entities: [constants_1.Entities.MERGE_REQUEST],
        relationships: [
            constants_1.Relationships.USER_APPROVED_PR,
            constants_1.Relationships.PROJECT_HAS_PR,
            constants_1.Relationships.USER_OPENED_PR,
        ],
        dependsOn: [constants_1.Steps.PROJECTS, constants_1.Steps.USERS],
        ingestionSourceId: constants_1.IngestionSources.MERGE_REQUESTS,
        executionHandler: fetchMergeRequests,
    },
];
//# sourceMappingURL=index.js.map