"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.projectLabelSteps = void 0;
exports.fetchProjectLabels = fetchProjectLabels;
const integration_sdk_core_1 = require("@jupiterone/integration-sdk-core");
const constants_1 = require("../../constants");
const provider_1 = require("../../provider");
const converter_1 = require("./converter");
async function fetchProjectLabels({ instance, jobState, logger, }) {
    const client = (0, provider_1.createAPIClient)(instance.config, logger);
    const failedProjects = new Set();
    await jobState.iterateEntities({ _type: constants_1.Entities.PROJECT._type }, async (projectEntity) => {
        const project = (0, integration_sdk_core_1.getRawData)(projectEntity);
        if (!project) {
            throw new integration_sdk_core_1.IntegrationError({
                code: 'RAW_DATA_MISSING',
                message: `Could not find Project entity with _key ${projectEntity._key}`,
            });
        }
        await client.iterateProjectLabels(project.fullPath, async (label) => {
            if (!label.id) {
                logger.warn({ id: label.id, title: label.title }, 'Label has no id');
                return;
            }
            const labelEntity = (0, converter_1.createLabelEntity)(label);
            if (!jobState.hasKey(labelEntity._key)) {
                await jobState.addEntity(labelEntity);
            }
            const projectLabelRelationship = (0, integration_sdk_core_1.createDirectRelationship)({
                _class: integration_sdk_core_1.RelationshipClass.HAS,
                from: projectEntity,
                to: labelEntity,
            });
            if (!jobState.hasKey(projectLabelRelationship._key)) {
                await jobState.addRelationship(projectLabelRelationship);
            }
        }, (err) => {
            if (err.errors.some((e) => e.includes(`The resource that you are attempting to access does not exist or you don't have permission to perform this action`))) {
                logger.warn(`Failed to fetch labels for project ${project.fullPath}. Errors: ${err.errors}`);
                failedProjects.add(project.fullPath);
            }
            else {
                throw err;
            }
        });
    });
    if (failedProjects.size > 0) {
        logger.publishWarnEvent({
            name: integration_sdk_core_1.IntegrationWarnEventName.MissingPermission,
            description: `Failed to fetch labels for ${Array.from(failedProjects).join(', ')}. Please ensure the API token has the correct permissions for those projects.`,
        });
    }
}
exports.projectLabelSteps = [
    {
        id: constants_1.Steps.PROJECT_LABELS,
        name: 'Fetch Project Labels',
        entities: [constants_1.Entities.LABEL],
        relationships: [constants_1.Relationships.PROJECT_HAS_LABEL],
        dependsOn: [constants_1.Steps.PROJECTS],
        ingestionSourceId: constants_1.IngestionSources.PROJECT_LABELS,
        executionHandler: fetchProjectLabels,
    },
];
//# sourceMappingURL=index.js.map