"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.userSteps = void 0;
exports.fetchUsers = fetchUsers;
const integration_sdk_core_1 = require("@jupiterone/integration-sdk-core");
const constants_1 = require("../../constants");
const provider_1 = require("../../provider");
const converter_1 = require("./converter");
const RESTClient_1 = require("../../provider/RESTClient");
const getNumericalId_1 = require("../../util/getNumericalId");
const promises_1 = require("timers/promises");
const ONE_MINUTE_IN_MS = 60_000;
async function fetchUsers({ instance, jobState, logger, }) {
    const client = (0, provider_1.createAPIClient)(instance.config, logger);
    const restClient = (0, RESTClient_1.createRESTClient)(instance.config, logger);
    const getDetails = async (userId) => {
        let details;
        try {
            const numericalId = (0, getNumericalId_1.getNumericalId)(userId);
            details = await restClient.getUserDetails(numericalId);
        }
        catch (err) {
            if (err.status === 429) {
                /**
                 * Wait 10 minutes, then attempt the request again.
                 *
                 * Also alert the user that we've apparently encountered a 10-minute
                 * rate limit on the `/users/<user-id>` endpoint, and guide them to
                 * documentation about increasing their per-user rate limit (if possible)
                 *
                 * See https://docs.gitlab.com/ee/user/admin_area/settings/rate_limit_on_users_api.html
                 */
                logger.publishWarnEvent({
                    name: 'warn_rate_limit_encountered',
                    description: `Encountered rate limit on '/users/${userId}' endpoint, which has a 10-minute rate limit reset (see https://docs.gitlab.com/ee/user/admin_area/settings/rate_limit_on_users_api.html). Sleeping for 10 minutes before re-attempting '/users/${userId}' endpoint`,
                });
                await (0, promises_1.setTimeout)(10 * ONE_MINUTE_IN_MS);
                return getDetails(userId);
            }
            else {
                logger.warn({ id: userId, error: err }, 'Failed to fetch user details for user');
            }
        }
        return details;
    };
    await jobState.iterateEntities(constants_1.Entities.GROUP, async (groupEntity) => {
        const group = (0, integration_sdk_core_1.getRawData)(groupEntity);
        if (!group) {
            throw new integration_sdk_core_1.IntegrationError({
                code: 'RAW_DATA_MISSING',
                message: `Could not find Group entity with _key ${groupEntity._key}`,
            });
        }
        if (!group.fullPath) {
            logger.warn({ group: groupEntity._key }, 'Group has no full path');
        }
        if (!group.groupMembersCount) {
            // skip groups with no members
            return;
        }
        try {
            await client.iterateGroupMembers(group.fullPath, async (member) => {
                if (!member.user) {
                    // Member connection does not have a user, shouldn't happen
                    return;
                }
                const userEntityKey = (0, converter_1.createUserEntityIdentifier)((0, getNumericalId_1.getNumericalId)(member.user.id));
                if (!jobState.hasKey(userEntityKey)) {
                    const userEntity = (0, converter_1.createUserEntity)(member.user, await getDetails(member.user.id));
                    await jobState.addEntity(userEntity);
                }
                await jobState.addRelationship((0, converter_1.createGroupHasUserRelationship)({
                    groupEntityKey: groupEntity._key,
                    userEntityKey,
                    edge: member,
                }));
            });
        }
        catch (err) {
            if (err instanceof integration_sdk_core_1.IntegrationProviderAuthorizationError) {
                logger.warn(`User does not have permission to fetch members of group ${group.id}. Please ensure this user has the right access type.`);
            }
            else {
                throw err;
            }
        }
    });
    await jobState.iterateEntities(constants_1.Entities.PROJECT, async (projectEntity) => {
        const project = (0, integration_sdk_core_1.getRawData)(projectEntity);
        if (!project) {
            throw new integration_sdk_core_1.IntegrationError({
                code: 'RAW_DATA_MISSING',
                message: `Could not find Project entity with _key ${projectEntity._key}`,
            });
        }
        try {
            await client.iterateProjectMembers(project.fullPath, async (member) => {
                if (!member.user) {
                    // Member connection does not have a user, shouldn't happen
                    return;
                }
                const userEntityKey = (0, converter_1.createUserEntityIdentifier)((0, getNumericalId_1.getNumericalId)(member.user.id));
                if (!jobState.hasKey(userEntityKey)) {
                    const userEntity = (0, converter_1.createUserEntity)(member.user, await getDetails(member.user.id));
                    await jobState.addEntity(userEntity);
                }
                const projectUserRelationship = (0, converter_1.createProjectHasUserRelationship)({
                    projectEntityKey: projectEntity._key,
                    userEntityKey,
                    edge: member,
                });
                if (!jobState.hasKey(projectUserRelationship._key)) {
                    await jobState.addRelationship(projectUserRelationship);
                }
            });
        }
        catch (err) {
            if (err instanceof integration_sdk_core_1.IntegrationProviderAuthorizationError) {
                logger.warn(`User does not have permission to fetch members of project ${project.id}. Please ensure this user has the right access type.`);
            }
            else {
                throw err;
            }
        }
    });
}
exports.userSteps = [
    {
        id: constants_1.Steps.USERS,
        name: 'Fetch users',
        entities: [constants_1.Entities.USER],
        relationships: [
            constants_1.Relationships.GROUP_HAS_USER,
            constants_1.Relationships.PROJECT_HAS_USER,
        ],
        dependsOn: [constants_1.Steps.GROUPS, constants_1.Steps.PROJECTS],
        ingestionSourceId: constants_1.IngestionSources.USERS,
        executionHandler: fetchUsers,
    },
];
//# sourceMappingURL=index.js.map